
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/ai/backend/logging:dist

from setuptools import setup

setup(**{
    'author': 'Lablup Inc. and contributors',
    'classifiers': [
        'Intended Audience :: Developers',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Environment :: No Input/Output (Daemon)',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development',
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python :: 3.12',
        'License :: OSI Approved :: MIT License',
    ],
    'description': 'Backend.AI Logging Subsystem',
    'install_requires': (
        'coloredlogs~=15.0',
        'graypy==2.1.0',
        'msgpack~=1.1.0',
        'python-json-logger~=3.2.0',
        'pyzmq~=26.2',
        'trafaret~=2.1',
        'yarl~=1.13.1',
    ),
    'license': 'MIT',
    'long_description': """Backend.AI Logging Subsystem
============================

Package Structure
-----------------

* `ai.backend.logging`
  - `abc`: Abstract base classes
  - `logger`: The core logging facility
    - `Logger`: The standard multiprocess-friendly logger using `RelayHandler` based on ZeroMQ
    - `LocalLogger`: A minimalized console/file logger that does not require serialization via networks at all
  - `handler`: Collection of vendor-specific handler implementations
  - `formatter`: Collection of formatters
  - `types`: Definition of enums/types like `LogLevel`
  - `utils`: Brace-style message formatting adapters and other extras
""",
    'long_description_content_type': 'text/markdown',
    'name': 'backend.ai-logging',
    'namespace_packages': (
    ),
    'package_data': {
        'ai.backend.logging': (
            'VERSION',
            'py.typed',
        ),
    },
    'packages': (
        'ai.backend.logging',
        'ai.backend.logging.handler',
    ),
    'project_urls': {
        'Documentation': 'https://docs.backend.ai/',
        'Source': 'https://github.com/lablup/backend.ai',
    },
    'python_requires': '>=3.12,<3.13',
    'url': 'https://github.com/lablup/backend.ai',
    'version': '25.3.0',
    'zip_safe': False,
})
